/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.activities;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.view.View;
import android.widget.ArrayAdapter;

import com.thingspace.cloud.sdk.enums.SortType;
import com.thingspace.cloud.sdk.list.ItemList;
import com.thingspace.cloud.sdk.list.item.Album;
import com.thingspace.cloud.sdk.list.item.File;
import com.thingspace.cloud.sdk.list.item.Item;
import com.thingspace.cloud.sdk.list.item.Playlist;
import com.thingspace.media.R;
import com.thingspace.media.adapters.RecyclerViewGridAdapter;
import com.thingspace.media.tasks.playlist.DeletePlaylistItemTask;
import com.thingspace.media.tasks.GetAlbumFilesTask;
import com.thingspace.media.tasks.playlist.GetPlaylistFilesTask;
import com.thingspace.media.tasks.HubbleMediaTask;

import java.util.ArrayList;

/**
 * AlbumsFilesActivity is called when an Album/Playlist is selected to display the files in them.
 * This Activity calls GetAlbumFilesTask and GetPlaylistFilesTask which makes a cloud SDK API call to retrieve contents of the Album/Playlist accordingly and also calls DeletePlaylistItemTask which makes a cloud SDK API call to delete a file from Album/Playlist.
 * When a photo is selected in album to view it, PhotoActivity is called which displays the full screen photo.
 */
public class AlbumsFilesActivity extends PlaylistItemsActivity {

    public static final String EXTRA_SOURCE_TYPE = "source_type";
    public static final int TYPE_ALBUM = 1;
    public static final int TYPE_PLAYLIST = 2;
    private static Album album;
    private static Playlist playlist;
    private ItemList<File> itemList;
    private int sourceType;

    public static void setAlbum(Album aAlbum) {
        album = aAlbum;
    }

    public static void setPlaylist(Playlist aPlaylist) {
        playlist = aPlaylist;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        sourceType = getIntent().getIntExtra(EXTRA_SOURCE_TYPE, TYPE_ALBUM);
        super.onCreate(savedInstanceState);

        if (sourceType == TYPE_ALBUM && contentType == CONTENT_TYPE_MUSIC) {
            floatingActionButton.setVisibility(View.GONE);
        }
    }

    protected void retrieveFiles() {
        switch (sourceType) {

            /**
             * GetAlbumFilesTask makes a Cloud SDK API call to Retrieve files from Photo or Music albums.
             */
            case TYPE_ALBUM:
                GetAlbumFilesTask getAlbumFilesTask = new GetAlbumFilesTask(this, this, album, SortType.NAME.setOrder(SortType.SortOrder.ASC));
                getAlbumFilesTask.execute();
                break;

            /**
             * GetPlaylistFilesTask makes a Cloud SDK API to retrieve files from an existing Playlist.
             */
            case TYPE_PLAYLIST:
                GetPlaylistFilesTask getPlaylistFilesTask = new GetPlaylistFilesTask(this, this, playlist, SortType.NAME.setOrder(SortType.SortOrder.ASC));
                getPlaylistFilesTask.execute();
        }
        showProgressDialog(getString(R.string.loading));
    }

    @Override
    public void onBindView(RecyclerView.ViewHolder holder, Item item) {
        if (item == null) {
            return;
        }

        File file = (File) item;
        loadImage(file.getMimeType(), file.getContentToken(), holder);
    }

    @Override
    public void onResponse(int taskCode, int responseCode, Object response) {
        if (response != null && (taskCode == HubbleMediaTask.TASK_ALBUM_FILES || taskCode == HubbleMediaTask.TASK_PLAYLIST_FILES)) {
            dismissProgressDialog();
            if (response instanceof ItemList) {
                itemList = (ItemList<File>) response;
                if (itemList.getCount() > 0) {
                    emptyView.setVisibility(View.GONE);
                    recyclerView.setVisibility(View.VISIBLE);
                    recyclerView.setAdapter(new RecyclerViewGridAdapter(this, R.layout.grid_item, itemList, cloudAPI, this, this));
                } else {
                    recyclerView.setVisibility(View.GONE);
                    emptyView.setVisibility(View.VISIBLE);
                    emptyView.setText(R.string.no_items);
                }
            }
            return;
        } else if (taskCode == HubbleMediaTask.TASK_ADD_TO_PLAYLIST) {
            /**
             * Updates the existing list as new files are added to the album/playlist.
             */
            if (response instanceof ArrayList) {
                ArrayList<String> returnedPaths = (ArrayList<String>) response;
                for (String path : returnedPaths) {
                    Log.d("===============>", "Added Path: " + path);
                }
                retrieveFiles();
            }
            return;
            /**
             * Updates the existing list as new files are deleted from the album/playlist.
             */
        } else if (response != null && ((boolean) response && taskCode == HubbleMediaTask.TASK_DELETE_PLAYLIST_ITEM)) {
            retrieveFiles();
            return;
        }

        super.onResponse(taskCode, responseCode, response);
    }

    protected String getPlaylistUid() {
        if (sourceType == TYPE_ALBUM) {
            return album.getUid();
        } else {
            return playlist.getUid();
        }
    }

    @Override
    public void onListItemClicked(View view, int position) {
        /**
         * When the photo is clicked, PhotoActivity is called which displays the full screen photo.
         */
        File file = itemList.getItem(position);
        String mimeType = file.getMimeType();
        if (mimeType != null && mimeType.indexOf("image") != -1) {
            Intent intent = new Intent(this, PhotoActivity.class);
            intent.putExtra(PhotoActivity.EXTRA_ITEM_NUMBER, position);
            PhotoActivity.setItemList(itemList);
            startActivity(intent);
        }
    }

    /**
     * Long press on a file prompts the user with an option to delete.
     */
    @Override
    public void onListItemLongClicked(View view, int position) {
        if (sourceType == TYPE_ALBUM && contentType == CONTENT_TYPE_MUSIC) {
            return;
        }

        File file = itemList.getItem(position);
        final String itemUid = file.getItemuid();
        AlertDialog.Builder operationDialog = new AlertDialog.Builder(this);
        final ArrayAdapter<String> arrayAdapter = new ArrayAdapter<>(this, R.layout.list_item);

        arrayAdapter.add(getString(R.string.action_delete));

        operationDialog.setAdapter(arrayAdapter, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                switch (which) {
                    case ACTION_DELETE:
                        AlertDialog.Builder deletePopup = new AlertDialog.Builder(AlbumsFilesActivity.this);
                        deletePopup.setMessage(R.string.delete_operation)
                                .setPositiveButton(getString(R.string.action_yes), new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        /**
                                         * DeletePlaylistItemTask makes a CLoud SDK API call to Delete a file from an existing Album/playlist.
                                         */
                                        DeletePlaylistItemTask task = new DeletePlaylistItemTask(AlbumsFilesActivity.this, AlbumsFilesActivity.this, cloudAPI, getPlaylistUid(), itemUid);
                                        task.execute();
                                    }
                                }).setNegativeButton(getString(R.string.action_no), null).create();
                        deletePopup.show();

                }
            }
        });

        operationDialog.setTitle(getString(R.string.file_actions));
        operationDialog.create();
        operationDialog.show();
    }
}
