/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.activities;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.widget.GridLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.text.TextUtils;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.callback.SearchCallback;
import com.thingspace.cloud.sdk.exception.SearchException;
import com.thingspace.cloud.sdk.list.ItemList;
import com.thingspace.cloud.sdk.list.item.FullviewItem;
import com.thingspace.cloud.sdk.list.item.Item;
import com.thingspace.media.R;
import com.thingspace.media.adapters.OnBindViewInterface;
import com.thingspace.media.adapters.RecyclerViewGridAdapter;
import com.thingspace.media.adapters.viewholders.RecyclerViewGridHolder;
import com.thingspace.media.listeners.RecyclerViewClickListener;
import com.thingspace.media.tasks.DownloadTask;
import com.thingspace.media.views.SpaceItemDecorator;

import java.util.ArrayList;

/**
 * This Activity performs the search and displays the results accordingly.
 */
public class SearchResultActivity extends BaseActivity implements RecyclerViewClickListener, OnBindViewInterface {

    public static final String EXTRA_QUERY = "extra_query";
    public static final String EXTRA_QUERY_TYPE = "extra_query_type";
    public static final String EXTRA_SORT_BY = "extra_sortBy";
    public static final String EXTRA_LIMIT = "extra_query_limit";
    public static final String EXTRA_TITLE = "extra_title";
    public static final String EXTRA_NAME = "extra_name";
    public static final String EXTRA_UID = "extra_uid";
    public static final String EXTRA_PLAYLIST_TYPE = "playlist_type";
    public static final String EXTRA_VIEW_MODE = "view_mode";
    public static final int VIEW_MODE_SELECTION = 11;
    public static final int VIEW_MODE_SELECTION_UPDATE = 12;
    public static final int VIEW_MODE_NORMAL = 0;

    public static final String RESULT_SELECTED_PATHS = "result_selected_paths";
    public static final String RESULT_SELECTED_NAMES = "result_selected_names";
    public static final String RESULT_SELECTED_URIS = "result_selected_uris";
    public static final String RESULT_SELECTED_SIZES = "result_selected_sizes";

    private static final int COLUMN_COUNT = 3;
    private static final int ACTION_DOWNLOAD = 0;

    private RecyclerViewGridAdapter gridCursorAdapter;
    private RecyclerView recyclerView;
    private String searchQuery;
    private String extraQueryType, extraSortBy, extraTitle;
    private int extraLimit = -1;
    private TextView emptyView;
    private ItemList itemListSearchResult;
    private int viewMode;
    private ArrayList<CharSequence> selectedPaths = new ArrayList<>();
    private ArrayList<CharSequence> selectedNames = new ArrayList<>();
    private ArrayList<CharSequence> selectedUris = new ArrayList<>();
    private ArrayList<CharSequence> selectedSizes = new ArrayList<>();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.resource_recycler_view);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        toolbar.setVisibility(View.VISIBLE);
        setSupportActionBar(toolbar);

        recyclerView = (RecyclerView) findViewById(R.id.recyclerView);
        emptyView = ((TextView) findViewById(R.id.emptyView));
        emptyView.setText(R.string.search_result_empty);
        searchQuery = getIntent().getStringExtra(EXTRA_QUERY);
        viewMode = getIntent().getIntExtra(EXTRA_VIEW_MODE, VIEW_MODE_NORMAL);

        if (viewMode == VIEW_MODE_SELECTION) {
            ImageView addImageView = (ImageView) findViewById(R.id.add_button);
            addImageView.setVisibility(View.VISIBLE);
            addImageView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (selectedPaths.size() == 0 && viewMode == VIEW_MODE_SELECTION) {
                        Toast.makeText(SearchResultActivity.this, "No item selected", Toast.LENGTH_SHORT).show();
                        return;
                    }

                    Intent intent = new Intent();
                    intent.putExtra(RESULT_SELECTED_PATHS, selectedPaths);
                    intent.putExtra(RESULT_SELECTED_NAMES, selectedNames);
                    intent.putExtra(RESULT_SELECTED_URIS, selectedUris);
                    intent.putExtra(RESULT_SELECTED_SIZES, selectedSizes);
                    intent.putExtra(EXTRA_NAME, getIntent().getStringExtra(EXTRA_NAME));
                    intent.putExtra(EXTRA_UID, getIntent().getStringExtra(EXTRA_UID));
                    intent.putExtra(EXTRA_PLAYLIST_TYPE, getIntent().getIntExtra(EXTRA_PLAYLIST_TYPE, 1));
                    setResult(RESULT_OK, intent);
                    finish();
                }
            });
        }

        GridLayoutManager manager = new GridLayoutManager(this, COLUMN_COUNT);
        manager.setSpanSizeLookup(new GridLayoutManager.SpanSizeLookup() {
            @Override
            public int getSpanSize(int position) {
                // for thumbs
                return 1;
            }
        });

        recyclerView.setLayoutManager(manager);
        recyclerView.setHasFixedSize(true);
        recyclerView.addItemDecoration(new SpaceItemDecorator(getResources().getDimensionPixelSize(R.dimen.thumb_spacing), 3));

        extraQueryType = getIntent().getStringExtra(EXTRA_QUERY_TYPE);
        extraSortBy = getIntent().getStringExtra(EXTRA_SORT_BY);
        extraLimit = getIntent().getIntExtra(EXTRA_LIMIT, -1);
        extraTitle = getIntent().getStringExtra(EXTRA_TITLE);
        if (!TextUtils.isEmpty(extraTitle)) {
            getSupportActionBar().setTitle(extraTitle);
        }

        Log.e(this.getClass().getSimpleName(), "Search Query: " + searchQuery);

        CloudAPI.QueryType queryType = CloudAPI.QueryType.SQL;
        if (extraQueryType != null && "Lucene".equalsIgnoreCase(extraQueryType)) {
            queryType = CloudAPI.QueryType.LUCENE;
        }

        cloudAPI.search(this, queryType, searchQuery, null, extraSortBy, extraLimit, new SearchCallback() {

            @Override
            public void onError(final SearchException exception) {
                /**
                 * Called when an error occurs during the search operation.
                 */
                Log.e("SEARCH RESULTS --->", "Search error: " + exception);
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        recyclerView.setVisibility(View.GONE);
                        emptyView.setVisibility(View.VISIBLE);
                        emptyView.setText("Search Exception: " + exception.getMessage());
                    }
                });
            }

            @Override
            public void onSuccess(final ItemList itemList, final boolean isOffline) {
                /**
                 * Called when the search completes successfully. The results of the search are provided to the {@link ItemList} object.
                 * Read the itemList and display the search results.
                 */
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        Log.e("SEARCH RESULTS --->", "search count: " + itemList.getCount() + " Offline: " + isOffline);
                        if (itemList.getCount() > 0) {
                            emptyView.setVisibility(View.GONE);
                            recyclerView.setVisibility(View.VISIBLE);
                        } else {
                            recyclerView.setVisibility(View.GONE);
                            emptyView.setVisibility(View.VISIBLE);
                            emptyView.setText(R.string.search_result_empty);
                        }

                        emptyView.setVisibility(itemList.getCount() > 0 ? View.GONE : View.VISIBLE);
                        gridCursorAdapter = new RecyclerViewGridAdapter(SearchResultActivity.this, R.layout.grid_item, itemList, cloudAPI, SearchResultActivity.this, SearchResultActivity.this);
                        recyclerView.setAdapter(gridCursorAdapter);
                        itemListSearchResult = itemList;

                        if (isOffline) {
                            Toast.makeText(SearchResultActivity.this, getString(R.string.search_offline_message), Toast.LENGTH_LONG).show();
                        }
                    }
                });
            }
        });
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        /**
         * If this activity is destroyed, close the itemListSearchResult as it is no longer needed.
         */
        if (itemListSearchResult != null) {
            itemListSearchResult.close();
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        return true;
    }

    @Override
    public void onListItemClicked(View view, int position) {
        FullviewItem fullviewItem = (FullviewItem) itemListSearchResult.getItem(position);

        CheckBox checkBox = (CheckBox) view.findViewById(R.id.checkbox);
        if (viewMode == VIEW_MODE_SELECTION) {
            String path = fullviewItem.getParentPath() + "/" + fullviewItem.getName();
            if (selectedPaths.contains(path)) {
                selectedPaths.remove(path);
                selectedNames.remove(fullviewItem.getName());
                selectedUris.remove(fullviewItem.getUri());
                selectedSizes.remove(fullviewItem.getSize());
                checkBox.setChecked(false);
            } else {
                selectedPaths.add(path);
                selectedNames.add(fullviewItem.getName());
                selectedUris.add(fullviewItem.getUri());
                selectedSizes.add(String.valueOf(fullviewItem.getSize()));
                checkBox.setChecked(true);
            }
        } else {
            String mimeType = fullviewItem.getMimeType();
            if (mimeType.indexOf("image") != -1) {
                Intent intent = new Intent(this, PhotoActivity.class);
                intent.putExtra(PhotoActivity.EXTRA_ITEM_NUMBER, position);
                PhotoActivity.setItemList(itemListSearchResult);
                startActivity(intent);
            }
        }
    }

    /**
     * When an item is long pressed, then it displays an option to download the file.
     * @param view     image view that was clicked
     * @param position position in the recycler view
     */
    @Override
    public void onListItemLongClicked(View view, int position) {
        final FullviewItem fullviewItem = (FullviewItem) itemListSearchResult.getItem(position);
        if (fullviewItem.getContentToken() == null) {
            return;
        }

        AlertDialog.Builder fileOperationDialog = new AlertDialog.Builder(this);
        final ArrayAdapter<String> arrayAdapter = new ArrayAdapter<>(this, R.layout.list_item);

        arrayAdapter.add(getString(R.string.action_download));

        fileOperationDialog.setAdapter(arrayAdapter, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                switch (which) {
                    case ACTION_DOWNLOAD:
                        /**
                         * DownloadTask makes an Cloud SDK API call to download a file from user's cloud storage account to specified location on the device
                         */
                        DownloadTask downloadTask = new DownloadTask(SearchResultActivity.this, SearchResultActivity.this, fullviewItem.getParentPath(), fullviewItem.getName(), cloudAPI);
                        downloadTask.execute();
                }
            }
        });

        fileOperationDialog.setTitle(getString(R.string.file_actions));
        fileOperationDialog.create();
        fileOperationDialog.show();
    }

    @Override
    public void onBindView(RecyclerView.ViewHolder holder, Item item) {
        if (viewMode == VIEW_MODE_SELECTION) {
            ((RecyclerViewGridHolder) holder).checkBox.setVisibility(View.VISIBLE);
        }

        FullviewItem fullviewItem = (FullviewItem) item;
        if (selectedPaths.contains(fullviewItem.getParentPath() + "/" + fullviewItem.getName())) {
            ((RecyclerViewGridHolder) holder).checkBox.setChecked(true);
        } else {
            ((RecyclerViewGridHolder) holder).checkBox.setChecked(false);
        }
    }
}
