/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.files.activity;

import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.GridView;
import android.widget.Toast;

import com.thingspace.cloud.sdk.list.ItemList;
import com.thingspace.cloud.sdk.list.item.MetadataItem;
import com.thingspace.files.R;
import com.thingspace.files.adapters.MetadataGridAdapter;
import com.thingspace.files.listeners.MetadataResponseListener;
import com.thingspace.files.listeners.OnCopyMoveResponseListener;
import com.thingspace.files.task.CopyFileFolderTask;
import com.thingspace.files.task.MetadataTask;
import com.thingspace.files.task.MoveFileFolderTask;
import com.thingspace.files.utils.KeplerUtil;

import java.net.HttpURLConnection;

/**
 * Activity to copy the file/folder and move Files/Folder to location.
 * This activity allows user to make a copy of file and store in different location, also allows user to move a file from present location to destination.
 * This activity calls {@link CopyFileFolderTask}, {@link MoveFileFolderTask}, {@link MetadataTask} which in turn calls Cloud SDK APIs to perform corresponding functionalities.
 */
public class CopyMoveActivity extends BaseActivity implements MetadataResponseListener, OnCopyMoveResponseListener {

    private static final String ROOT_CALL_PATH = "/";
    private static final String MEDIA_TYPE_IMAGE = "image";
    private static final int ACTION_COPY = 0;
    private static final int ACTION_MOVE = 1;
    private static String currentFolderOpened = "";
    private String srcFileFolderPath = "";
    private int copyMoveAction = 0;
    private ProgressDialog dialog;
    private GridView gridView;
    private MetadataGridAdapter metaAdapter;
    private ItemList<MetadataItem> metadataResponse;
    private boolean fromDataClass = false;

    private AdapterView.OnItemClickListener folderClick = new AdapterView.OnItemClickListener() {
        @Override
        public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
            MetadataItem item = metadataResponse.getItem(position);
            if (item.getContentToken() != null) {
                if (item.getMimeType().contains(MEDIA_TYPE_IMAGE)) {

                }
            } else {
                startMetadataTask("/".equals(item.getParentPath()) ? item.getParentPath() + item.getName()
                        : item.getParentPath() + "/" + item.getName());
            }
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        findViewById(R.id.clickablelayout).setVisibility(View.VISIBLE);
        srcFileFolderPath = getIntent().getStringExtra(FileBrowserMainActivity.EXTRA_COPY_MOVE_SRC);
        copyMoveAction = getIntent().getIntExtra(FileBrowserMainActivity.EXTRA_COPY_MOVE_ACTION, 0);
        fromDataClass = getIntent().getBooleanExtra(DataClassActivity.EXTRA_COPY_MOVE_CLASS_TYPE, false);

        gridView = (GridView) findViewById(R.id.gridView);
        if (ACTION_MOVE == copyMoveAction) {
            ((Button) findViewById(R.id.move_copy_button)).setText(getString(R.string.move_here));
        }
    }

    protected void onAuthorizationComplete() {
        startMetadataTask(ROOT_CALL_PATH);
    }

    private void startMetadataTask(String path) {
        showBusyDialog(true);
        currentFolderOpened = path;

        /**
         * Fetches metadata details of a file/folder in path specified by making Cloud SDK API call.
         */
        MetadataTask task = new MetadataTask(this, path, cloudAPI);
        task.execute();
    }

    public void onCopyMoveCancelClick(View v) {
        //cancel the copy/move action
        finish();
    }

    public void onCopyMoveButtonClick(View v) {
        if (currentFolderOpened.equalsIgnoreCase(ROOT_CALL_PATH)) {
            Toast.makeText(CopyMoveActivity.this, getString(R.string.cannot_perform_action), Toast.LENGTH_LONG).show();
            return;
        }

        if (fromDataClass) {
            setResult(copyMoveAction, new Intent().putExtra("target", currentFolderOpened));
            finish();
        } else {
            if (!KeplerUtil.isNetworkAvailable(this)) {
                Toast.makeText(this, getString(R.string.error_unavailable), Toast.LENGTH_SHORT).show();
                return;
            }

            if (copyMoveAction == ACTION_COPY) {
                /**
                 * Copy file/folder to the target location by making a cloud SDK API call.
                 */
                CopyFileFolderTask task = new CopyFileFolderTask(CopyMoveActivity.this, CopyMoveActivity.this, cloudAPI, srcFileFolderPath, currentFolderOpened);
                showBusyDialog(true);
                task.execute();
            } else if (copyMoveAction == ACTION_MOVE) {
                /**
                 * Move file/folder to the location by making a cloud SDK API call
                 */
                MoveFileFolderTask task = new MoveFileFolderTask(CopyMoveActivity.this, CopyMoveActivity.this, cloudAPI, srcFileFolderPath, currentFolderOpened);
                showBusyDialog(true);
                task.execute();
            }
        }
    }

    /**
     * Called when Metadata API response in received.
     *
     * @param response     List of file/folder response for cloud folder
     * @param responseCode HTTP Response code
     */
    @Override
    public void onMetadataResponseReceived(ItemList response, int responseCode) {
        showBusyDialog(false);

        if (responseCode == HttpURLConnection.HTTP_OK) {
            this.metadataResponse = response;
            metaAdapter = new MetadataGridAdapter(this, cloudAPI, R.layout.grid_item, response);
            gridView.setAdapter(metaAdapter);
            gridView.setOnItemClickListener(folderClick);
        } else if (responseCode == HttpURLConnection.HTTP_UNAVAILABLE) {
            Toast.makeText(CopyMoveActivity.this, getString(R.string.error_unavailable), Toast.LENGTH_LONG).show();
            return;
        } else if (responseCode == HttpURLConnection.HTTP_UNAUTHORIZED) {
            Toast.makeText(CopyMoveActivity.this, getString(R.string.error_unathorized), Toast.LENGTH_LONG).show();
            cloudAPI.logout(this);
            Intent intent = new Intent(CopyMoveActivity.this, FileBrowserMainActivity.class);
            intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
            startActivity(intent);
        } else {
            Toast.makeText(CopyMoveActivity.this, getString(R.string.error_happened), Toast.LENGTH_LONG).show();
        }
    }

    @Override
    public void onGetFileResponseReceived(String response, int responseCode) {

    }

    /**
     * Called when Copy file/folder API response in received.
     *
     * @param response     List of file/folder response for copy
     * @param responseCode HTTP Response code
     */
    @Override
    public void onCopyResponseReceived(ItemList response, int responseCode) {
        showBusyDialog(false);
        if (responseCode == HttpURLConnection.HTTP_UNAUTHORIZED) {
            Toast.makeText(CopyMoveActivity.this, getString(R.string.auth_failed), Toast.LENGTH_LONG).show();
            setResult(responseCode);
            finish();
            return;
        }

        if (responseCode == HttpURLConnection.HTTP_OK && response != null) {
            Toast.makeText(CopyMoveActivity.this, getString(R.string.toast_copy_message), Toast.LENGTH_LONG).show();
        } else {
            Toast.makeText(CopyMoveActivity.this, getString(R.string.error_happened), Toast.LENGTH_LONG).show();
        }
        setResult(responseCode);
        finish();
    }

    /**
     * Called when Move file/folder API response in received.
     *
     * @param response     List of file/folder response for move
     * @param responseCode HTTP Response code
     */
    @Override
    public void onMoveResponseReceived(ItemList response, int responseCode) {
        showBusyDialog(false);
        if (responseCode == HttpURLConnection.HTTP_UNAUTHORIZED) {
            Toast.makeText(CopyMoveActivity.this, getString(R.string.auth_failed), Toast.LENGTH_LONG).show();
            setResult(responseCode);
            finish();
            return;
        }

        if (responseCode == HttpURLConnection.HTTP_OK && response != null) {
            Toast.makeText(CopyMoveActivity.this, getString(R.string.toast_move_message), Toast.LENGTH_LONG).show();
        } else {
            Toast.makeText(CopyMoveActivity.this, getString(R.string.error_happened), Toast.LENGTH_LONG).show();
        }
        setResult(responseCode);
        finish();
    }

    public void showBusyDialog(boolean show) {
        if (dialog == null && show) {
            dialog = new ProgressDialog(CopyMoveActivity.this);
            dialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
            dialog.setMessage("Loading. Please wait...");
            dialog.setIndeterminate(true);
            dialog.setCanceledOnTouchOutside(false);
            dialog.show();
        } else if (dialog != null && !show) {
            dialog.dismiss();
            dialog = null;

        }

    }
}